//
// tu58 - Emulate a TU58 over a serial line
//
// Original (C) 1984 Dan Ts'o <Rockefeller Univ. Dept. of Neurobiology>
// Update (C) 2005 Don North <ak6dn_at_mindspring_dot_com>
//
// This is the TU58 emulation program written at Rockefeller Univ., Dept. of
// Neurobiology. We copyright (C) it and permit its use provided it is not
// sold to others. Originally written by Dan Ts'o circa 1984 or so.
//
// Extensively rewritten to work reliably under the CYGWIN unix subsystem.
// Has been tested up to 38.4Kbaud hosted on a 1GHz Pentium3 Win2K platform.
// Added native windows serial comm support compile option (for break detect).
// Added multithreaded supervisor for command parser and emulator restart.
// Added timing delay option to allow emulator to pass diagnostic ZTUUF0.BIN.
// Supports four (or more, compile option) logical TU58 devices per controller.
//
// 12 Jul 2005 - donorth - Initial rewrite
// 22 Feb 2006 - donorth - Updated for linux compatibility
//

//
// TU58 Emulator Common Definitions
//


// Common includes

#include <unistd.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <time.h>

#ifndef O_BINARY
#define O_BINARY 0		// for linux compatibility
#endif



// Constants

#define NTU58		4	// number of devices to emulate (0..N-1)

#define TAPESIZE	512	// number of blocks per tape
#define BLOCKSIZE	512	// number of bytes per block

#define FILEREAD	1	// file can be read
#define FILEWRITE	2	// file can be written
#define FILECREATE	3	// file should be created
#define FILERT11INIT	4	// file should be init'ed as RT11 structure
#define FILEXXDPINIT	5	// file should be init'ed as XXDP structure

#define DEV_NYI		-1	// not yet implemented
#define DEV_OK		0	// no error
#define DEV_BREAK	1	// BREAK on line
#define DEV_ERROR	2	// ERROR on line



// Prototypes

// main.c
void fatal (char *, ...);
void error (char *, ...);
void info (char *, ...);

// serial.c
void devtxstop (void);
void devtxstart (void);
void devtxinit (void);
void devtxflush (void);
void devtxput (unsigned char);
int  devtxwrite (unsigned char *, int);
void devrxinit (void);
int  devrxavail (void);
int  devrxpeek (void);
int  devrxerror (void);
unsigned char devrxget (void);
void devinit (char *, int);
void devrestore (void);
void coninit (void);
void conrestore (void);
int  conget (void);

// file.c
void fileinit (void);
int  fileopen (char *, int);
int  fileunit (int);
int  fileseek (int, int, int, int);
int  fileread (int, unsigned char *, int);
int  filewrite (int, unsigned char *, int);
void fileclose (void);

// tu58.c
void tu58 (void);


// Globals

extern int errno;
extern int debug;
extern int xoffen;
extern int mrspen;
extern int timing;
extern int verbose;


// the end
